import { RootNode, TemplateChildNode, ParentNode, ExpressionNode, JSChildNode, CacheExpression, ElementNode, NodeTransform, DirectiveTransform, CompilerError, BindingMetadata, SimpleExpressionNode, CompoundExpressionNode, InterpolationNode, TextNode, TransformContext as TransformContext$1, ParserOptions as ParserOptions$1, CodegenResult } from '@vue/compiler-core';
export { DirectiveNode, DirectiveTransform, NodeTransform, SimpleExpressionNode, TransformContext, findProp } from '@vue/compiler-core';
import { ParserPlugin } from '@babel/parser';
import { GeneratorOptions } from '@babel/generator';
import { Expression, Identifier, Pattern, RestElement, ObjectProperty, SpreadElement } from '@babel/types';
import { MiniProgramCompilerOptions } from '@fe6/qinjs-cli-shared';

/** @format */
declare class IdentifierGenerator {
    private _chars;
    private _nextIds;
    next(): string;
    _increment(): void;
    [Symbol.iterator](): Generator<string, void, unknown>;
}

/** @format */

interface ImportItem {
    exp: string | ExpressionNode;
    path: string;
}
declare const enum BindingComponentTypes {
    SELF = "self",
    SETUP = "setup",
    UNKNOWN = "unknown"
}
interface TransformContext extends Required<Omit<TransformOptions, 'filename' | 'root'>> {
    selfName: string | null;
    currentNode: RootNode | TemplateChildNode | null;
    parent: ParentNode | null;
    childIndex: number;
    helpers: Map<symbol, number>;
    components: Set<string>;
    imports: ImportItem[];
    bindingComponents: Record<string, {
        type: BindingComponentTypes;
        name: string;
    }>;
    identifiers: {
        [name: string]: number | undefined;
    };
    cached: number;
    scopes: {
        vFor: number;
        vueId: number;
    };
    scope: CodegenRootScope;
    currentScope: CodegenScope;
    currentVueId: string;
    vueIds: string[];
    inVOnce: boolean;
    inVFor: boolean;
    helper<T extends symbol>(name: T): T;
    removeHelper<T extends symbol>(name: T): void;
    helperString(name: symbol): string;
    replaceNode(node: TemplateChildNode): void;
    removeNode(node?: TemplateChildNode): void;
    onNodeRemoved(): void;
    addIdentifiers(exp: ExpressionNode | string): void;
    removeIdentifiers(exp: ExpressionNode | string): void;
    popScope(): CodegenScope | undefined;
    getScopeIndex(scope: CodegenScope): number;
    addVIfScope(initScope: CodegenVIfScopeInit): CodegenVIfScope;
    addVForScope(initScope: CodegenVForScopeInit): CodegenVForScope;
    cache<T extends JSChildNode>(exp: T, isVNode?: boolean): CacheExpression | T;
    isMiniProgramComponent(name: string): 'plugin' | 'component' | 'dynamicLib' | undefined;
    rootNode: TemplateChildNode | null;
}

type VForOptions = Omit<ForParseResult, 'tagType'> & {
    sourceExpr?: Expression;
    sourceAlias: string;
    sourceCode: string;
    valueCode: string;
    valueExpr: Identifier | Pattern | RestElement;
    valueAlias: string;
    keyCode: string;
    keyExpr: Identifier | Pattern | RestElement;
    keyAlias: string;
    indexCode: string;
    indexExpr: Identifier | Pattern | RestElement;
    indexAlias: string;
    node: ElementNode;
};
type ForElementNode = ElementNode & {
    vFor: VForOptions & {
        source: ExpressionNode;
    };
};
declare function isForElementNode(node: unknown): node is ForElementNode;
interface ForParseResult {
    source: ExpressionNode;
    value: ExpressionNode;
    key: ExpressionNode;
    index: ExpressionNode;
    tagType: any;
}

/** @format */

interface ErrorHandlingOptions {
    onWarn?: (warning: CompilerError) => void;
    onError?: (error: CompilerError) => void;
}
interface ParserOptions {
    /**
     * e.g. platform native elements, e.g. `<div>` for browsers
     */
    isNativeTag?: (tag: string) => boolean;
    /**
     * e.g. native elements that can self-close, e.g. `<img>`, `<br>`, `<hr>`
     */
    isVoidTag?: (tag: string) => boolean;
    /**
     * Separate option for end users to extend the native elements list
     */
    isCustomElement?: (tag: string) => boolean | void;
}
interface SharedTransformCodegenOptions {
    inline?: boolean;
    isTS?: boolean;
    root?: string;
    filename?: string;
    bindingMetadata?: BindingMetadata;
    prefixIdentifiers?: boolean;
    miniProgram?: MiniProgramCompilerOptions;
}
interface TransformOptions extends SharedTransformCodegenOptions, ErrorHandlingOptions {
    hashId?: string | null;
    scopeId?: string | null;
    filters?: string[];
    renderDataSpread?: boolean;
    cacheHandlers?: boolean;
    nodeTransforms?: NodeTransform[];
    directiveTransforms?: Record<string, DirectiveTransform | undefined>;
    isBuiltInComponent?: (tag: string) => symbol | void;
    isCustomElement?: (tag: string) => boolean | void;
    expressionPlugins?: ParserPlugin[];
    skipTransformIdentifier?: boolean;
    bindingCssVars?: string[];
}
interface CodegenRootScope {
    id: IdentifierGenerator;
    identifiers: string[];
    properties: (ObjectProperty | SpreadElement)[];
    parent: CodegenScope | null;
}
interface CodegenVIfScopeInit {
    name: 'if' | 'else-if' | 'else' | string;
    condition?: Expression;
}
type CodegenVForScopeInit = VForOptions & {
    locals: string[];
};
interface CodegenVIfScope extends CodegenRootScope, CodegenVIfScopeInit {
    parentScope: CodegenRootScope | CodegenVForScope;
}
interface CodegenVForScope extends CodegenRootScope, CodegenVForScopeInit {
}
type CodegenScope = CodegenRootScope | CodegenVIfScope | CodegenVForScope;
interface CodegenOptions extends SharedTransformCodegenOptions {
    mode?: 'module' | 'function';
    scopeId?: string | null;
    sourceMap?: boolean;
    runtimeModuleName?: string;
    runtimeGlobalName?: string;
    generatorOpts?: GeneratorOptions;
    dir?: string;
}
type CompilerOptions = ParserOptions & TransformOptions & CodegenOptions;

/** @format */

type CodegenNode = SimpleExpressionNode | CompoundExpressionNode | InterpolationNode | TextNode;
interface GenNodeContext {
    code: string;
    helper(key: symbol): string;
    push(code: string, node?: CodegenNode): void;
}
declare function genExpr(node: CodegenNode | symbol | string, context?: GenNodeContext): string;

/** @format */

declare function rewriteExpression(node: ExpressionNode, context: TransformContext | TransformContext$1, babelNode?: Expression, scope?: CodegenScope, { property, ignoreLiteral, referencedScope, }?: {
    property: boolean;
    ignoreLiteral: boolean;
    referencedScope?: CodegenScope;
}): SimpleExpressionNode;

/** @format */

declare const transformOn: DirectiveTransform;

/** @format */

declare const transformModel: DirectiveTransform;

/** @format */
declare const V_ON: unique symbol;
declare const V_FOR: unique symbol;
declare const EXTEND: unique symbol;
declare const SET_REF: unique symbol;
declare const CAMELIZE: unique symbol;
declare const HYPHENATE: unique symbol;
declare const RENDER_PROPS: unique symbol;
declare const RENDER_SLOT: unique symbol;
declare const DYNAMIC_SLOT: unique symbol;
declare const WITH_SCOPED_SLOT: unique symbol;
declare const STRINGIFY_STYLE: unique symbol;
declare const NORMALIZE_CLASS: unique symbol;
declare const TO_DISPLAY_STRING: unique symbol;
declare const WITH_MODEL_MODIFIERS: unique symbol;

/** @format */

declare function parse(template: string, options?: ParserOptions$1): RootNode;
declare function beforeCompile(theDir: string): (template: string, options?: CompilerOptions) => CodegenResult;

export { CAMELIZE, type CompilerOptions, DYNAMIC_SLOT, EXTEND, HYPHENATE, NORMALIZE_CLASS, RENDER_PROPS, RENDER_SLOT, SET_REF, STRINGIFY_STYLE, TO_DISPLAY_STRING, V_FOR, V_ON, WITH_MODEL_MODIFIERS, WITH_SCOPED_SLOT, beforeCompile, genExpr, isForElementNode, parse, rewriteExpression, transformModel, transformOn };
